if [ -n "$CCLUB_CONFIG_DASH_X" ]; then
    set -x
fi

CCLUB_SCRIPTS="/usr/share/cclub-scripts"

CCLUB_CONFIG_STATE="/var/lib/cclub-configuration"
CCLUB_CONFIG_DCHASH_DIR="$CCLUB_CONFIG_STATE/dchash"
CCLUB_CONFIG_FLAGS_DIR="$CCLUB_CONFIG_STATE/flags"
CCLUB_CONFIG_GENERATED_DIR="$CCLUB_CONFIG_STATE/generated"
CCLUB_CONFIG_UCF_DIR="$CCLUB_CONFIG_STATE/ucf"
CCLUB_CONFIG_SHARE="/usr/share/cclub-configuration"

if [ -n "$CCLUB_CONFIG_DRY_RUN" ]; then
    dry_run=echo
fi


debug_echo()
{
    if [ -n "$CCLUB_CONFIG_VERBOSE" ]; then
        echo "$@" >&2
    fi
}

is_flag_set()
{
    local flags="$1"
    local desired="$2"

    echo "$flags" | grep -q '\(^\|,\)'"$desired"'\(,\|$\)'
}

is_package_installed()
{
    local package="$1"

    set +e
    state=`dpkg-query -f '${db:Status-Abbrev}' -W "$package" 2>/dev/null`
    code=$?
    set -e

    if [ $code -eq 0 ]; then
        case "$state" in
        i[iwt]*)
            return 0
            ;;
        esac
    fi
    return 1
}

quote()
{
    /bin/echo -E "$1" | sed -e "s/'/'\\\\''/g" -e "s/^/'/" -e "s/$/'/"
}

get_tmp_file()
{
    mktemp --tmpdir cclub-configuration.XXXXXXXX
}

get_current_debconf_configuration()
{
    local preseed="$1"

    local patterns=`get_tmp_file`
    grep -v -e '^[ \t]*#' -e '^[ \t]*$' "$preseed" |
        awk -- 'BEGIN { OFS="\t" } // { print "",$2,"" }' > $patterns

    local current=`get_tmp_file`
    debconf-get-selections | fgrep -f "$patterns" | sort -k 1,2 > "$current" 
    rm -f "$patterns"

    echo "$current"
}

get_adduser_configuration()
{
    eval `sed -e '/^[ \t]*\(FIRST\|LAST\)_SYSTEM_[UG]ID=[0-9]\+[ \t]*$/{'   \
              -e     's/^[ \t]\+//'                                         \
              -e     's/^/CONF_/'                                           \
              -e     'b'                                                    \
              -e '}'                                                        \
              -e 'd' /etc/adduser.conf || true`
}

configure_via_copy()
{
    local target="$1"
    local new="$2"
    local flags="$3"

    debug_echo " ---- configure_via_copy for $target"

    local cmd="ucf --three-way --state-dir=`quote "$CCLUB_CONFIG_UCF_DIR"`"

    if [ "${CCLUB_CONFIG_VERBOSE:-0}" -ge 3 ]; then
        cmd="$cmd --verbose"
    fi

    if is_flag_set "$flags" force; then
        debug_echo "Force flag is set; clobbering \"$target\""
        cmd="env -u UCF_FORCE_CONFFOLD UCF_FORCE_CONFFNEW=1 $cmd"
    fi

    if is_flag_set "$flags" generated; then
        cmd="$cmd --src-dir=`quote "$CCLUB_CONFIG_GENERATED_DIR"`"
    fi

    $dry_run eval "$cmd `quote "$new"` `quote "$target"`"
}

configure_via_debconf_advanced()
{
    local name="$1"
    local current="$2"
    local preseed="$3"
    local flags="$4"

    debug_echo " ---- configure_via_debconf_advanced for $name"

    local hashfile="$CCLUB_CONFIG_DCHASH_DIR/$name"
    local historyfile="${preseed}.md5sum"

    local cursum=
    if [ `stat --format=%s "$current"` -ne 0 ]; then
        cursum=`md5sum "$current" | awk -- '// { print $1 }'`
        debug_echo "Current \"$name\" debconf settings have md5sum \"$cursum\""
        if [ "${CCLUB_CONFIG_VERBOSE:-0}" -ge 2 ]; then
            echo '>>>>>>' 1>&2
            cat "$current" 1>&2
            echo '<<<<<<' 1>&2
        fi
    fi

    local doit=
    if is_flag_set "$flags" force; then
        debug_echo "Force flag is set; clobbering \"$name\" debconf settings"
        doit=1
    elif [ -z "$cursum" ]; then
        debug_echo "No current \"$name\" debconf settings; will preseed"
        doit=1
    elif [ -f "$hashfile" ]; then
        local prevsum=`cat "$hashfile"`
        debug_echo "Previous version resulted in md5sum \"$prevsum\""
        if [ "$cursum" = "$prevsum" ]; then
            debug_echo "Current and previous md5sums match; will update"
            doit=1
        else
            debug_echo "Current and previous md5sums differ; will NOT update"
        fi
    elif [ -f "$historyfile" ]; then
        if fgrep -q -i "$cursum" "$historyfile"; then
            debug_echo "Current md5sum is a historical version; will update"
            doit=1
        else
            debug_echo "Current md5sum is unknown; will NOT update"
        fi
    fi

    local canonical=`get_tmp_file`
    grep -v -e '^[ \t]*#' -e '^[ \t]*$' -e '[ \t]seen\([ \t]\|$\)' "$preseed" |
        sed -e '# Get rid of any leading or trailing whitespace' \
            -e 's/^[ \t]*//' -e 's/[ \t]*$//' \
            -e '# Turn the 1st, 2nd, and 3rd sequences of whitespace into' \
            -e '# single tab characters' \
            -e 's/[ \t]\+/\t/1' -e 's/[ \t]\+/\t/2' -e 's/[ \t]\+/\t/3' |
        sort -k 1,2 > "$canonical"
    newsum=`md5sum "$canonical" | awk -- '// { print $1 }'`
    $dry_run eval "echo `quote "$newsum"` > `quote "$hashfile"`"
    debug_echo "New \"$name\" debconf settings have md5sum \"$newsum\""
    if [ "${CCLUB_CONFIG_VERBOSE:-0}" -ge 2 ]; then
        echo '>>>>>>' 1>&2
        cat "$canonical" 1>&2
        echo '<<<<<<' 1>&2
    fi
    rm -f "$canonical"

    if [ -n "$doit" ]; then
        debug_echo "Applying new debconf settings"
        $dry_run eval "debconf-set-selections < `quote "$preseed"`"
        package=`echo "$name" | sed -e 's/^dc://' -e 's/~.*$//'`
        if is_flag_set "$flags" reconfigure &&
           is_package_installed "$package"; then
            $dry_run dpkg-reconfigure -fnoninteractive "$package"
        fi
    else
        debug_echo "Retaining old debconf settings"
    fi
}

configure_via_debconf()
{
    local name="$1"
    local preseed="$2"
    local flags="$3"

    local current=`get_current_debconf_configuration "$preseed"`
    configure_via_debconf_advanced "$name" "$current" "$preseed" "$flags"
    rm -f "$current"
}
